package com.koutra.dist.proc.cluster;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class ClusterFuture implements Future<Object> {
	private Request request;
	private CountDownLatch latch;
	private Object result;
	
	public ClusterFuture(Request request) {
		this.request = request;
		this.latch = new CountDownLatch(1);
		this.result = null;
	}
	
	public Request getRequest() {
		return request;
	}
	
	public void setResult(Object result) {
		this.result = result;
		this.latch.countDown();
	}

	@Override
	public boolean cancel(boolean mayInterruptIfRunning) {
		// Not really used in the clustering case, so resorting to the simplest
		// implementation.
		return false;
	}

	@Override
	public Object get() throws InterruptedException, ExecutionException {
		latch.await();
		return result;
	}

	@Override
	public Object get(long timeout, TimeUnit unit) throws InterruptedException,
			ExecutionException, TimeoutException {
		if (!latch.await(timeout, unit))
			throw new TimeoutException(
					"Timeout expired while waiting for a cluster future result.");
		return result;
	}

	@Override
	public boolean isCancelled() {
		// Not really used in the clustering case, so resorting to the simplest
		// implementation.
		return false;
	}

	@Override
	public boolean isDone() {
		return latch.getCount() == 0;
	}

}
