/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.pipeline;

import java.io.BufferedReader;
import java.io.Reader;

import org.apache.log4j.Logger;

import com.koutra.dist.proc.model.ContentType;
import com.koutra.dist.proc.model.IDemuxFaucet;
import com.koutra.dist.proc.model.IPipelineItem;
import com.koutra.dist.proc.model.ISink;
import com.koutra.dist.proc.model.ISinkTemplate;
import com.koutra.dist.proc.model.XformationException;
import com.koutra.dist.proc.util.ExecutorsHelper;

public abstract class LineBasedReaderDemuxPipelineItem extends
		ReaderDemuxPipelineItem implements IDemuxFaucet {
	private static final Logger logger = Logger.getLogger(LineBasedReaderDemuxPipelineItem.class);
	
	protected BufferedReader bufferedReader;

	/**
	 * @deprecated Use any of the initializing constructors instead.
	 */
	public LineBasedReaderDemuxPipelineItem() { }
	
	public LineBasedReaderDemuxPipelineItem(String id,
			ISinkTemplate pipelineTemplate) {
		super(id, pipelineTemplate);
		this.bufferedReader = null;
	}

	@Override
	public char[] transformBuffer(char[] buffer, int off, int len) {
		return null;
	}

	/**
	 * Transform a single line of input.
	 * @param line The input line or null if the end of the input has been reached.
	 * @return The transformed String to pass down the pipeline chain.
	 */
	public abstract String transformLine(String line);
	
	@Override
	public Object consume(ISink previousSink) {
		if (!hookedUpFaucet)
			throw new XformationException("Pipeline item has not been set up correctly:" +
					"faucet has not been set");
		
		if (previousSink instanceof ISinkTemplate) {
			// If the sink is a clone we must stop the consumption chain here, as the
			// rest of the chain has already been notified the first time around.
			if (((ISinkTemplate) previousSink).isClone())
				return null;
		}
		
	    if (faucet instanceof IPipelineItem)
	    	((IPipelineItem) faucet).consume(this);

		createPipeline();

		reader = (Reader) faucet.getSource(ContentType.CharStream);
		bufferedReader = new BufferedReader(reader);
		
		if (logger.isTraceEnabled())
			logger.trace("Created bufferedReader: " + bufferedReader +
					" wrapping reader " + reader);

		getSource(ContentType.CharStream);
		
		if (sink.get() instanceof IPipelineItem)
			((IPipelineItem) sink.get()).handleMuxSwitch();
		
		final ISink fPreviousSink = previousSink; // needed in order to be used below
		
		ExecutorsHelper.getInstance().executeInProc(new Runnable() {
			@Override
			public void run() {
				try {
					String line;
					if (logger.isTraceEnabled())
						logger.trace("Using reader " + bufferedReader + " for line based demux reader");
					while ((line = bufferedReader.readLine()) != null) {
						String transformedLine = transformLine(line);
						if (logger.isTraceEnabled())
							logger.trace("Read line: '" + line +
									"' from " + bufferedReader + ". Writing '" +
									transformedLine + "' to: " + writer);
						writer.write(transformedLine);
						while (shouldChangePipeline()) {
							writer.write("");
							writer.close();
							if (logger.isTraceEnabled())
								logger.trace("About to switch pipeline");
							switchPipeline();
							line = transformLine("");
						}
					}
					writer.write(transformLine(line));
					writer.close();
					
					// Send a null down to the mux sink so that it knows that the ball
					// has stopped rolling.
					if (fPreviousSink instanceof IPipelineItem) {
						IPipelineItem pipelineSink = (IPipelineItem) fPreviousSink;
						pipelineSink.endMux();
					}
					
					logger.debug("Executable for the line reader pipeline will now exit.");
				} catch (Throwable t) {
					logger.error("Error during pipeline thread execution.", t);
				}
			}
		});
		
		return null;
	}
}
