/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.pipeline;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.UUID;

import com.koutra.dist.proc.model.ContentType;
import com.koutra.dist.proc.model.FaucetAdapter;
import com.koutra.dist.proc.model.IDemuxFaucet;
import com.koutra.dist.proc.model.IFaucet;
import com.koutra.dist.proc.model.IFaucetTemplate;
import com.koutra.dist.proc.model.IMuxSink;
import com.koutra.dist.proc.model.IPipelineItem;
import com.koutra.dist.proc.model.IPipelineItemTemplate;
import com.koutra.dist.proc.model.ISimplePipelineItem;
import com.koutra.dist.proc.model.ISimplePipelineItemTemplate;
import com.koutra.dist.proc.model.ISink;
import com.koutra.dist.proc.model.ISinkTemplate;
import com.koutra.dist.proc.model.XformationException;

public abstract class ReaderToXMLPipelineItemTemplate extends ReaderToXMLPipelineItem
		implements ISimplePipelineItemTemplate {

	protected boolean isClone;

	public ReaderToXMLPipelineItemTemplate() {
		super(UUID.randomUUID().toString());
		isClone = false;
	}
	
	@Override
	public boolean isClone() {
		return isClone;
	}

	@Override
	public void createClone(IPipelineItem pipelineItem) {
		try {
			ReaderToXMLPipelineItemTemplate clone = getClass().newInstance();
			clone.id = id;
			clone.isClone = true;
			clone.hookedUpFaucet = false;
			clone.hookedUpSink = false;
			clone.sink = null;
			clone.faucet = null;
			clone.reader = null;
			
			if (pipelineItem instanceof ISimplePipelineItem)
				((ISimplePipelineItem) pipelineItem).hookupSink(clone);
			else if (pipelineItem instanceof FaucetAdapter)
				((FaucetAdapter) pipelineItem).hookupSink(clone);
			
			if (sink instanceof ISinkTemplate) {
				((ISinkTemplate) sink).createClone(clone);
			} else {
				sink.hookupFaucet(clone);
			}
		} catch (Exception e) {
			throw new XformationException("Unable to create clone", e);
		}
	}

	@Override
	protected void checkFaucetValidity(IFaucet faucet) {
		if (!faucet.supportsOutput(ContentType.CharStream))
			throw new IllegalArgumentException("Faucet '" + faucet.getId() +
					"' must support the CharStream content type.");
		if (!(faucet instanceof IDemuxFaucet) &&
				!(faucet instanceof IFaucetTemplate) &&
				!(faucet instanceof FaucetAdapter))
			throw new IllegalArgumentException("Faucet '" + faucet.getId() +
					"' is neither an IDemuxFaucet nor an IFaucetTemplate instance, " +
					"nor a FaucetAdapter. " +
					"This is not allowed.");
	}

	@Override
	protected void checkSinkValidity(ISink sink) {
		if (!sink.supportsInput(ContentType.XML))
			throw new IllegalArgumentException("Sink '" + sink.getId() +
					"' must support the XML content type.");
		if (!(sink instanceof IMuxSink) && !(sink instanceof ISinkTemplate))
			throw new IllegalArgumentException("Sink '" + sink.getId() +
					"' is neither an IMuxSink or an ISinkTemplate instance. " +
					"This is not allowed.");
	}

	@Override
	public void handleMuxSwitch() {
		if (sink instanceof IPipelineItem)
			((IPipelineItem) sink).handleMuxSwitch();
		else
			sink.registerSource(this);
	}

	@Override
	public void endMux() {
		if (sink instanceof IPipelineItemTemplate) {
			((IPipelineItemTemplate) sink).endMux();
		} else {
			sink.registerSource(null);
		}
	}
	
	@Override
	public void writeTo(DataOutputStream out) throws IOException {
		super.writeTo(out);
		out.writeBoolean(isClone);
	}

	@Override
	public void readFrom(DataInputStream in) throws IOException,
			IllegalAccessException, InstantiationException {
		super.readFrom(in);
		this.isClone = in.readBoolean();
	}
	
}
