/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.util;

import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

public class DumpingInputStream extends InputStream {
	private static final Logger logger = Logger.getLogger(DumpingInputStream.class);
	
	private InputStream delegate;
	
	public DumpingInputStream(InputStream delegate) {
		if (logger.isTraceEnabled())
			logger.trace("Created dumping input stream");
		this.delegate = delegate;
	}

	@Override
	public int read() throws IOException {
		int byteRead = delegate.read();
		
		if (logger.isTraceEnabled()) {
			if (byteRead == -1) {
				logger.trace("Read EOF.");
			} else {
				byte[] buffer = new byte[1];
				buffer[0] = (byte) byteRead;
				logger.trace("Read: " + ByteArrayUtil.getFullHexDump(buffer, 0, 1));
				logger.trace("Read: " + new String(buffer, 0, 1));
			}
		}
		
		return byteRead;
	}
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		int bytesRead = delegate.read(b, off, len);
		
		if (logger.isTraceEnabled()) {
			if (bytesRead == -1) {
				logger.trace("Read EOF.");
			} else {
				logger.trace("Read: " + ByteArrayUtil.getFullHexDump(b, off, bytesRead));
				logger.trace("Read: " + new String(b, off, bytesRead));
			}
		}
		
		return bytesRead;
	}
	
	@Override
	public void close() throws IOException {
		if (logger.isTraceEnabled())
			logger.trace("Closing input stream.");
		delegate.close();
	}

}
