/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.util;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

public class EngineHelper {
	private static final EngineHelper singleton = new EngineHelper();
	
	public static EngineHelper getInstance() {
		return singleton;
	}
	
	protected ScriptEngineManager manager;
	protected Map<String, ScriptEngine> keyToEngineMap;
	
	private EngineHelper() {
		manager = new ScriptEngineManager();
		keyToEngineMap = new HashMap<String, ScriptEngine>();
	}
	
	protected String getKey(String scriptEngineName, String path) {
		return scriptEngineName + "|" + path;
	}
	
	public synchronized ScriptEngine evalScript(String scriptEngineName, String path)
			throws FileNotFoundException, ScriptException {
		ScriptEngine retVal = keyToEngineMap.get(getKey(scriptEngineName, path));
		if (retVal == null) {
			retVal = manager.getEngineByName(scriptEngineName);
			retVal.eval(new FileReader(path));
			keyToEngineMap.put(getKey(scriptEngineName, path), retVal);
		}
		return retVal;
	}
}
