/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.xml;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;

public class SAXElement {
	
	private int position;
	private AttributesImpl attributes;
	private String uri;
	private String localName;
	private String qName;
	private List<Object> parameters;
	
	public SAXElement() {
		this.position = 0;
		this.attributes = null;
		this.uri = null;
		this.localName = null;
		this.qName = null;
		this.parameters = new ArrayList<Object>();
	}
	
	public SAXElement(String uri, String localName, String qName, Attributes attributes) {
		this.position = 0;
		this.uri = uri;
		this.localName = localName;
		this.qName = qName;
		this.parameters = new ArrayList<Object>();
		
		setAttributes(attributes);
	}

	public void setPosition(int position) {
		this.position = position;
	}

	public int getPosition() {
		return position;
	}
	
	public void setAttributes(Attributes attributes) {
		// Copy the attributes object into a new AttributesImpl object 
		this.attributes = new AttributesImpl(attributes);
	}
	
	public AttributesImpl getAttributes() {
		return attributes;
	}
	
	public String getUri() {
		return uri;
	}
	
	public String getLocalName() {
		return localName;
	}
	
	public String getQName() {
		return qName;
	}
	
	public static String getName(String localName, String qName) {
		return (localName != null && !localName.equals(""))?localName:qName;		
	}
	
	public String getName() {
		return getName(localName, qName);
	}
	
	public void addParameter(Object parameter) {
		parameters.add(parameter);
	}

	public List<Object> getParameters() {
		return parameters;
	}
	
	public String toString() {
		return getName();
	}

}
