/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.xml;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class TraversedPath {

	private Stack<SAXElement> stack = new Stack<SAXElement>();
	private ElementStackPath path = new ElementStackPath();
	
	public int getDepth() {
		return stack.size();
	}
	
	public SAXElement getLeaf() {
		if (stack != null && stack.size() > 0)
			return stack.lastElement();
		else
			return null;
	}
	
	public SAXElement getElement(int level) {
		return stack.get(level);
	}

	public void startDocument() {
		path.startDocument();
	}
	
	public void endDocument() throws SAXException {
		 if (stack.size() != 0)
			throw new SAXException(
					"Malformed document (ended at " + stack.toString() + ")");
	}
	
	public int getCurrentDepth() {
		return stack.size();
	}
	
	public SAXElement startElement(String uri, String localName, 
			String qName, Attributes atts) {
		SAXElement se = new SAXElement(uri, localName, qName, atts);

		stack.push(se);
		path.push(SAXElement.getName(localName, qName));
		
		return se;
	}
	
	public SAXElement endElement(String uri, String localName, String qName) {
		SAXElement retVal = getLeaf();
		stack.pop();
		path.pop();
		return retVal;
	}
	
	public boolean compareTo(String xPath, int matchingLevel) {
		return path.compareTo(xPath, matchingLevel);
	}
	
	public boolean compareTo(String nextElement, String xPath, int matchingLevel) {
		path.push(nextElement);
		boolean retValue = compareTo(xPath, matchingLevel+1);
		path.pop();
		return retValue;
	}

	public Iterator<SAXElement> getElementsIterator() {
		ArrayList<SAXElement> al = new ArrayList<SAXElement>(stack);
		return al.iterator();
	}
	
	public Iterator<SAXElement> getElementsReverseIterator() {
		ArrayList<SAXElement> al = new ArrayList<SAXElement>(stack);
		Collections.reverse(al);
		return al.iterator();
	}

	public String toString() {
		return path.toString();
	}

}
